@@ -15,13 +15,17 @@
 */
 
 /* Initially written by David Mattes (david.mattes@boeing.com) */
+/* Portuguese eID card support by Joao Poupino (joao.poupino@ist.utl.pt) */
+
+#include "config.h"
 
-#include "internal.h"
-#include "cardctl.h"
-#include "asn1.h"
 #include <stdlib.h>
 #include <string.h>
 
+#include "internal.h"
+#include "asn1.h"
+#include "cardctl.h"
+
 static struct sc_card_operations gemsafe_ops;
 static struct sc_card_operations *iso_ops = NULL;
 
@@ -32,20 +36,31 @@ static struct sc_card_driver gemsafe_drv = {
 	NULL, 0, NULL
 };
 
-static const char *gemexpresso_atrs[] = {
-	/* standard version    */
-	"3B:7B:94:00:00:80:65:B0:83:01:01:74:83:00:90:00",
-	"3B:6B:00:00:80:65:B0:83:01:01:74:83:00:90:00",
-	/* fips 140 version    */
-	"3B:6B:00:00:80:65:B0:83:01:03:74:83:00:90:00",
-	/* TODO: add more ATRs */
-	"3B:7A:94:00:00:80:65:A2:01:01:01:3D:72:D6:43",
-	"3B:7D:94:00:00:80:31:80:65:B0:83:01:01:90:83:00:90:00",
-	NULL
+/* Known ATRs */
+static struct sc_atr_table gemsafe_atrs[] = {
+	/* standard version */
+    {"3B:7B:94:00:00:80:65:B0:83:01:01:74:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_GENERIC, 0, NULL},
+    {"3B:6B:00:00:80:65:B0:83:01:01:74:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_GENERIC, 0, NULL},
+    /* GemSafeXpresso 32K */
+    {"3b:6d:00:00:80:31:80:65:b0:83:01:02:90:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_GENERIC, 0, NULL},
+    /* fips 140 version */
+    {"3B:6B:00:00:80:65:B0:83:01:03:74:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_GENERIC, 0, NULL},
+    /* Undefined */
+    {"3B:7A:94:00:00:80:65:A2:01:01:01:3D:72:D6:43", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_GENERIC, 0, NULL},
+    {"3B:7D:94:00:00:80:31:80:65:B0:83:01:01:90:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_GENERIC, 0, NULL},
+    /* Portuguese eID cards */
+    {"3B:7D:95:00:00:80:31:80:65:B0:83:11:C0:A9:83:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_PTEID, 0, NULL},
+    {"3B:7D:95:00:00:80:31:80:65:B0:83:11:C0:A9:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_PTEID, 0, NULL},
+    {"3B:7D:95:00:00:80:31:80:65:B0:83:11:00:C8:83:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_PTEID, 0, NULL},
+    {"3B:7D:95:00:00:80:31:80:65:B0:83:11:00:C8:83:00:90:00", NULL, NULL, SC_CARD_TYPE_GEMSAFEV1_PTEID, 0, NULL},
+    {NULL, NULL, NULL, 0, 0, NULL}
 };
 
 static const u8 gemsafe_def_aid[] = {0xA0, 0x00, 0x00, 0x00, 0x18, 0x0A,
 	0x00, 0x00, 0x01, 0x63, 0x42, 0x00};
+
+static const u8 gemsafe_pteid_aid[] = {0x60, 0x46, 0x32, 0xFF, 0x00, 0x00, 0x02};
+
 /*
 static const u8 gemsafe_def_aid[] = {0xA0, 0x00, 0x00, 0x00, 0x63, 0x50,
 	0x4B, 0x43, 0x53, 0x2D, 0x31, 0x35};
@@ -63,7 +78,7 @@ static int get_conf_aid(sc_card_t *card, u8 *aid, size_t *len)
 	int			i;
 	const char		*str_aid;
 
-	SC_FUNC_CALLED(ctx, 1);
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
 	conf_block = NULL;
 	for (i = 0; ctx->conf_blocks[i] != NULL; i++) {
@@ -75,13 +90,13 @@ static int get_conf_aid(sc_card_t *card, u8 *aid, size_t *len)
 	}
 
 	if (!conf_block) {
-		sc_debug(ctx, "no card specific options configured, trying default AID\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "no card specific options configured, trying default AID\n");
 		return SC_ERROR_INTERNAL;
 	}
 
 	str_aid = scconf_get_str(conf_block, "aid", NULL);
 	if (!str_aid) {
-		sc_debug(ctx, "no aid configured, trying default AID\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "no aid configured, trying default AID\n");
 		return SC_ERROR_INTERNAL;
 	}
 	return sc_hex_to_bin(str_aid, aid, len);
@@ -94,7 +109,7 @@ static int gp_select_applet(sc_card_t *card, const u8 *aid, size_t aid_len)
 	struct sc_context *ctx = card->ctx;
 	struct sc_apdu    apdu;
 
-	SC_FUNC_CALLED(ctx, 1);
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0xa4, 0x04, 0x00);
 	apdu.lc      = aid_len;
@@ -105,35 +120,20 @@ static int gp_select_applet(sc_card_t *card, const u8 *aid, size_t aid_len)
 	apdu.resplen = sizeof(buf);
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(ctx, r, "APDU transmit failed");
+	SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r)
-		SC_FUNC_RETURN(ctx, 2, r);
+		SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 
 	return SC_SUCCESS;
 }
 
-static int gemsafe_match_card(struct sc_card *card)
+static int gemsafe_match_card(sc_card_t *card)
 {
-	int i, match = -1;
+	int i;
 
-	SC_FUNC_CALLED(card->ctx, 1);
-
-	for (i = 0; gemexpresso_atrs[i] != NULL; i++) {
-		u8 defatr[SC_MAX_ATR_SIZE];
-		size_t len = sizeof(defatr);
-		const char *atrp = gemexpresso_atrs[i];
-
-		if (sc_hex_to_bin(atrp, defatr, &len))
-			continue;
-		if (len != card->atr_len)
-			continue;
-		if (memcmp(card->atr, defatr, len) != 0)
-			continue;
-		match = i + 1;
-		break;
-    	}
-	if (match == -1)
+	i = _sc_match_atr(card, gemsafe_atrs, &card->type);
+	if (i < 0)
 		return 0;
 
 	return 1;
@@ -144,7 +144,7 @@ static int gemsafe_init(struct sc_card *card)
 	int	r;
 	gemsafe_exdata *exdata = NULL;
 
-	SC_FUNC_CALLED(card->ctx, 1);
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
 	card->name = "GemSAFE V1";
 	card->cla  = 0x00;
@@ -153,12 +153,17 @@ static int gemsafe_init(struct sc_card *card)
 	if (!exdata)
 		return SC_ERROR_OUT_OF_MEMORY;
 	exdata->aid_len = sizeof(exdata->aid);
-	/* try to get a AID from the config file */
-	r = get_conf_aid(card, exdata->aid, &exdata->aid_len);
-	if (r < 0) {
-		/* failed, use default value */
-		memcpy(exdata->aid, gemsafe_def_aid, sizeof(gemsafe_def_aid));
-		exdata->aid_len = sizeof(gemsafe_def_aid);
+	if(card->type == SC_CARD_TYPE_GEMSAFEV1_GENERIC) {
+		/* try to get a AID from the config file */
+		r = get_conf_aid(card, exdata->aid, &exdata->aid_len);
+		if (r < 0) {
+			/* failed, use default value */
+			memcpy(exdata->aid, gemsafe_def_aid, sizeof(gemsafe_def_aid));
+			exdata->aid_len = sizeof(gemsafe_def_aid);
+		}
+	} else if (card->type == SC_CARD_TYPE_GEMSAFEV1_PTEID) {
+		memcpy(exdata->aid, gemsafe_pteid_aid, sizeof(gemsafe_pteid_aid));
+		exdata->aid_len = sizeof(gemsafe_pteid_aid);
 	}
 
 	/* increase lock_count here to prevent sc_unlock to select
@@ -168,7 +173,7 @@ static int gemsafe_init(struct sc_card *card)
 	r = gp_select_applet(card, exdata->aid, exdata->aid_len);
 	if (r < 0) {
 		free(exdata);
-		sc_debug(card->ctx, "applet selection failed\n");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "applet selection failed\n");
 		return SC_ERROR_INTERNAL;
 	}
 	card->lock_count--;
@@ -207,7 +212,7 @@ static int gemsafe_select_file(struct sc_card *card, const struct sc_path *path,
 	   struct sc_file **file_out)
 {
 	/* so far just call the iso select file (but this will change) */
-	SC_FUNC_CALLED(card->ctx, 1);
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
 	return iso_ops->select_file(card, path, file_out);
 }
@@ -251,8 +256,8 @@ static int gemsafe_setacl(sc_card_t *card, sc_file_t *file, const u8 *data,
 			cond = *p++;
 		else
 			cond = 0xff;
-		if(ctx->debug >= 3)
-			sc_debug(ctx, "DF security byte CREATE DF: %02x\n", cond);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"DF security byte CREATE DF: %02x\n", cond);
 		r = gemsafe_sc2acl(file, SC_AC_OP_CREATE, cond);
 		if (r < 0)
 			return r;
@@ -260,8 +265,8 @@ static int gemsafe_setacl(sc_card_t *card, sc_file_t *file, const u8 *data,
 			cond = *p;
 		else
 			cond = 0xff;
-		if(ctx->debug >= 3)
-			sc_debug(ctx, "DF security byte CREATE EF: %02x\n", cond);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"DF security byte CREATE EF: %02x\n", cond);
 		/* XXX: opensc doesn't currently separate access conditions for
 		 * CREATE EF and CREATE DF, this should be changed */
 		r = gemsafe_sc2acl(file, SC_AC_OP_CREATE, cond);
@@ -274,8 +279,8 @@ static int gemsafe_setacl(sc_card_t *card, sc_file_t *file, const u8 *data,
 			cond = *p++;
 		else
 			cond = 0xff;
-		if(ctx->debug >= 3)
-			sc_debug(ctx, "EF security byte UPDATE/ERASE BINARY: %02x\n", cond);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"EF security byte UPDATE/ERASE BINARY: %02x\n", cond);
 		r = gemsafe_sc2acl(file, SC_AC_OP_UPDATE, cond);
 		if (r < 0)
 			return r;
@@ -289,8 +294,8 @@ static int gemsafe_setacl(sc_card_t *card, sc_file_t *file, const u8 *data,
 			cond = *p;
 		else
 			cond = 0xff;
-		if(ctx->debug >= 3)
-			sc_debug(ctx, "EF security byte READ BINARY: %02x\n", cond);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"EF security byte READ BINARY: %02x\n", cond);
 		r = gemsafe_sc2acl(file, SC_AC_OP_READ, cond);
 		if (r < 0)
 			return r;
@@ -308,13 +313,13 @@ static int gemsafe_process_fci(struct sc_card *card, struct sc_file *file,
 	const char *type;
 	struct sc_context *ctx = card->ctx;
 
-	SC_FUNC_CALLED(ctx, 1);
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
 	r = iso_ops->process_fci(card, file, buf, len);
 	if (r < 0)
 		return r;
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "processing GemSAFE V1 specific FCI information\n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+		"processing GemSAFE V1 specific FCI information\n");
 
 
 	tag = sc_asn1_find_tag(ctx, p, len, 0x82, &tlen);
@@ -327,34 +332,33 @@ static int gemsafe_process_fci(struct sc_card *card, struct sc_file *file,
 		file->type = SC_FILE_TYPE_WORKING_EF;
 	}
 
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "file type: %s\n", type);
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "file type: %s\n", type);
 
 	tag = sc_asn1_find_tag(ctx, p, len, 0x8C, &tlen);
 	if (tag) {
 		r = gemsafe_setacl(card, file, tag, strcmp(type, "DF") ? 0 : 1);
 		if (r < 0) {
-			sc_debug(ctx, "unable to set ACL\n");
+			sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "unable to set ACL\n");
 			return SC_ERROR_INTERNAL;
 		}
 	} else
-		sc_debug(ctx, "error: AM and SC bytes missing\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "error: AM and SC bytes missing\n");
 
 	return SC_SUCCESS;
 }
 
-static u8 gemsafe_flags2algref(const struct sc_security_env *env)
+static u8 gemsafe_flags2algref(struct sc_card *card, const struct sc_security_env *env)
 {
 	u8 ret = 0;
 
 	if (env->operation == SC_SEC_OPERATION_SIGN) {
 		if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1)
-			ret = 0x12;
+			ret = card->type == SC_CARD_TYPE_GEMSAFEV1_PTEID ? 0x02 : 0x12;
 		else if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_ISO9796)
 			ret = 0x11;
 	} else if (env->operation == SC_SEC_OPERATION_DECIPHER) {
 		if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1)
-			ret = 0x12;
+			ret = card->type == SC_CARD_TYPE_GEMSAFEV1_PTEID ? 0x02 : 0x12;
 	}
 
 	return ret;
@@ -365,12 +369,12 @@ static int gemsafe_restore_security_env(struct sc_card *card, int se_num)
 	int r;
 	struct sc_apdu apdu;
 
-	SC_FUNC_CALLED(card->ctx, 1);
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_1, 0x22, 0x73, (u8) se_num);
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);
 }
@@ -384,18 +388,18 @@ static int gemsafe_set_security_env(struct sc_card *card,
 	struct sc_security_env se_env = *env;
 	struct sc_context *ctx = card->ctx;
 
-	SC_FUNC_CALLED(ctx, 1);
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
 	if (!(se_env.flags & SC_SEC_ENV_ALG_REF_PRESENT)) {
 		/* set the algorithm reference */
-		alg_ref = gemsafe_flags2algref(&se_env);
+		alg_ref = gemsafe_flags2algref(card, &se_env);
 		if (alg_ref) {
 			se_env.algorithm_ref = alg_ref;
 			se_env.flags |= SC_SEC_ENV_ALG_REF_PRESENT;
 		}
 	}
 	if (!(se_env.flags & SC_SEC_ENV_ALG_REF_PRESENT))
-		sc_debug(ctx, "unknown algorithm flags '%x'\n", se_env.algorithm_flags);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "unknown algorithm flags '%x'\n", se_env.algorithm_flags);
 
 	se_env.flags &= ~SC_SEC_ENV_FILE_REF_PRESENT;
 	return iso_ops->set_security_env(card, &se_env, se_num);
@@ -404,24 +408,29 @@ static int gemsafe_set_security_env(struct sc_card *card,
 static int gemsafe_compute_signature(struct sc_card *card, const u8 * data,
 	size_t data_len, u8 * out, size_t outlen)
 {
-	int r;
+	int r, len;
 	struct sc_apdu apdu;
 	u8 rbuf[SC_MAX_APDU_BUFFER_SIZE];
 	u8 sbuf[SC_MAX_APDU_BUFFER_SIZE];
 	sc_context_t *ctx = card->ctx;
 
-	SC_FUNC_CALLED(ctx, 1);
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
 	if (data_len > 36) {
-		sc_debug(ctx, "error: input data too long: %lu bytes\n", data_len);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "error: input data too long: %lu bytes\n", data_len);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0x2A, 0x9E, 0xAC);
-	apdu.cla |= 0x80;
-	apdu.resp = rbuf;
-	apdu.resplen = sizeof(rbuf);
-	apdu.le      = 256;
+	/* the Portuguese eID card requires a two-phase exchange */
+	if(card->type == SC_CARD_TYPE_GEMSAFEV1_PTEID) {
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0x2A, 0x90, 0xA0);
+	} else {
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0x2A, 0x9E, 0xAC);
+		apdu.cla |= 0x80;
+		apdu.resp = rbuf;
+		apdu.resplen = sizeof(rbuf);
+		apdu.le      = 256;
+	}
 	/* we sign a digestInfo object => tag 0x90 */
 	sbuf[0] = 0x90;
 	sbuf[1] = (u8)data_len;
@@ -429,17 +438,27 @@ static int gemsafe_compute_signature(struct sc_card *card, const u8 * data,
 	apdu.data = sbuf;
 	apdu.lc   = data_len + 2;
 	apdu.datalen = data_len + 2;
-	apdu.sensitive = 1;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
-		int len = apdu.resplen > outlen ? outlen : apdu.resplen;
+		if(card->type == SC_CARD_TYPE_GEMSAFEV1_PTEID) {
+			/* finalize the exchange */
+			sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0x2A, 0x9E, 0x9A);
+			apdu.le = 128; /* 1024 bit keys */
+			apdu.resp = rbuf;
+			apdu.resplen = sizeof(rbuf);
+			r = sc_transmit_apdu(card, &apdu);
+			SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+			if(apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
+		}
+		len = apdu.resplen > outlen ? outlen : apdu.resplen;
 
 		memcpy(out, apdu.resp, len);
-		SC_FUNC_RETURN(card->ctx, 4, len);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, len);
 	}
-	SC_FUNC_RETURN(card->ctx, 2, sc_check_sw(card, apdu.sw1, apdu.sw2));
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
 
 static int gemsafe_decipher(struct sc_card *card, const u8 * crgram,
@@ -450,29 +469,47 @@ static int gemsafe_decipher(struct sc_card *card, const u8 * crgram,
 	u8 rbuf[SC_MAX_APDU_BUFFER_SIZE];
 	sc_context_t *ctx = card->ctx;
 
-	SC_FUNC_CALLED(ctx, 1);
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 	if (crgram_len > 255)
-		SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0x2A, 0x80, 0x84);
 	apdu.cla |= 0x80;
 	apdu.resp = rbuf;
 	apdu.resplen = sizeof(rbuf);
 	apdu.le      = crgram_len;
-	apdu.sensitive = 1;
 
 	apdu.data = crgram;
 	apdu.lc   = crgram_len;
 	apdu.datalen = crgram_len;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		int len = apdu.resplen > outlen ? outlen : apdu.resplen;
 
 		memcpy(out, apdu.resp, len);
-		SC_FUNC_RETURN(card->ctx, 2, len);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, len);
 	}
-	SC_FUNC_RETURN(card->ctx, 2, sc_check_sw(card, apdu.sw1, apdu.sw2));
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
+}
+
+static int gemsafe_get_challenge(sc_card_t *card, u8 *rnd, size_t len)
+{
+	int prev_cla, r;
+
+	prev_cla = card->cla;
+	if(card->type == SC_CARD_TYPE_GEMSAFEV1_PTEID) {
+		/* Warning: this depends on iso7816_get_challenge not
+		 * changing the value of the card's CLA
+		 */
+		card->cla = 0x80;
+	}
+	r = iso_ops->get_challenge(card, rnd, len);
+	/* Restore the CLA value if needed */
+	if(card->cla != prev_cla)
+		card->cla = prev_cla;
+
+	return r;
 }
 
 static int gemsafe_build_pin_apdu(struct sc_card *card,
@@ -549,7 +586,6 @@ static int gemsafe_build_pin_apdu(struct sc_card *card,
 	apdu->datalen = len;
 	apdu->data = sbuf;
 	apdu->resplen = 0;
-	apdu->sensitive = 1;
 
 	return 0;
 }
@@ -586,19 +622,17 @@ static int gemsafe_pin_cmd(struct sc_card *card, struct sc_pin_cmd_data *data,
 		/* Call the reader driver to collect
 		 * the PIN and pass on the APDU to the card */
 		if (data->pin1.offset == 0) {
-			sc_error(card->ctx,
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
 				"Card driver didn't set PIN offset");
 			return SC_ERROR_INVALID_ARGUMENTS;
 		}
 		if (card->reader
 		 && card->reader->ops
 		 && card->reader->ops->perform_verify) {
-			r = card->reader->ops->perform_verify(card->reader,
-					card->slot,
-					data);
+			r = card->reader->ops->perform_verify(card->reader, data);
 			/* sw1/sw2 filled in by reader driver */
 		} else {
-			sc_error(card->ctx,
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
 				"Card reader driver does not support "
 				"PIN entry through reader key pad");
 			r = SC_ERROR_NOT_SUPPORTED;
@@ -609,7 +643,7 @@ static int gemsafe_pin_cmd(struct sc_card *card, struct sc_pin_cmd_data *data,
 	if (data->apdu == &local_apdu)
 		data->apdu = NULL;
 
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu->sw1 == 0x63) {
 		if ((apdu->sw2 & 0xF0) == 0xC0 && tries_left != NULL)
 			*tries_left = apdu->sw2 & 0x0F;
@@ -625,7 +659,7 @@ static struct sc_card_driver *sc_get_driver(void)
 		iso_ops = iso_drv->ops;
 	/* use the standard iso operations as default */
 	gemsafe_ops = *iso_drv->ops;
-	/* gemsafe specfic functions */
+	/* gemsafe specific functions */
 	gemsafe_ops.match_card	= gemsafe_match_card;
 	gemsafe_ops.init	= gemsafe_init;
 	gemsafe_ops.finish	= gemsafe_finish;
@@ -634,6 +668,7 @@ static struct sc_card_driver *sc_get_driver(void)
 	gemsafe_ops.set_security_env     = gemsafe_set_security_env;
 	gemsafe_ops.decipher             = gemsafe_decipher;
 	gemsafe_ops.compute_signature    = gemsafe_compute_signature;
+	gemsafe_ops.get_challenge 		 = gemsafe_get_challenge;
 	gemsafe_ops.process_fci	= gemsafe_process_fci;
 	gemsafe_ops.pin_cmd		 = gemsafe_pin_cmd;
 
