@@ -66,7 +66,7 @@ static int rtecp_init(sc_card_t *card)
 	unsigned long flags;
 
 	assert(card && card->ctx);
-	card->caps |= SC_CARD_CAP_NO_FCI | SC_CARD_CAP_RNG;
+	card->caps |= SC_CARD_CAP_RNG;
 	card->cla = 0;
 
 	flags = SC_ALGORITHM_RSA_RAW | SC_ALGORITHM_ONBOARD_KEY_GEN
@@ -365,7 +365,7 @@ static int rtecp_cipher(sc_card_t *card, const u8 *data, size_t data_len,
 	apdu.datalen = data_len;
 	apdu.resp = buf_out;
 	apdu.resplen = out_len + 2;
-	apdu.le = out_len;
+	apdu.le = out_len > 256 ? 256 : out_len;
 	if (apdu.lc > 255)
 		apdu.flags |= SC_APDU_FLAGS_CHAINING;
 	r = sc_transmit_apdu(card, &apdu);
@@ -383,8 +383,8 @@ static int rtecp_cipher(sc_card_t *card, const u8 *data, size_t data_len,
 		if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		{
 			assert(buf_out);
-			for (i = 0; i < out_len && i < apdu.resplen; ++i)
-				out[i] = buf_out[out_len - 1 - i];
+			for (i = 0; i < apdu.resplen; ++i)
+				out[i] = buf_out[apdu.resplen - 1 - i];
 			r = (i > 0) ? (int)i : SC_ERROR_INTERNAL;
 		}
 		else
@@ -486,12 +486,36 @@ static int rtecp_reset_retry_counter(sc_card_t *card, unsigned int type,
 	sc_apdu_t apdu;
 	int r;
 
-	(void)type, (void)puk, (void)puklen, (void)newref, (void)newlen; /* no warning */
+	(void)type, (void)puk, (void)puklen; /* no warning */
 	assert(card && card->ctx);
+
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_1, 0x2C, 0x03, ref_qualifier);
 	r = sc_transmit_apdu(card, &apdu);
 	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Unblock card failed");
+	
+	if (newref && newlen)   {
+        	u8 tmp[2], buf[SC_MAX_APDU_BUFFER_SIZE];
+		u8 *p = buf;
+
+		tmp[0] = (newlen >> 8) & 0xFF;
+		tmp[1] = newlen & 0xFF;
+		sc_asn1_put_tag(0x80, tmp, sizeof(tmp), p, sizeof(buf) - (p - buf), &p);
+		r = sc_asn1_put_tag(0xA5, newref, newlen, p, sizeof(buf) - (p - buf), &p);
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Invalid new PIN length");
+
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0x24, 0x01, ref_qualifier);
+		apdu.lc = p - buf;
+		apdu.data = buf;
+		apdu.datalen = p - buf;
+
+		r = sc_transmit_apdu(card, &apdu);
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+		r = sc_check_sw(card, apdu.sw1, apdu.sw2);
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Set PIN failed");
+	}
+
 	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 }
 
