@@ -29,6 +29,8 @@
 #include "asn1.h"
 #include "cardctl.h"
 
+#define _FINEID_BROKEN_SELECT_FLAG 1
+
 static struct sc_atr_table setcos_atrs[] = {
 	/* some Nokia branded SC */
 	{ "3B:1F:11:00:67:80:42:46:49:53:45:10:52:66:FF:81:90:00", NULL, NULL, SC_CARD_TYPE_SETCOS_GENERIC, 0, NULL },
@@ -48,6 +50,10 @@ static struct sc_atr_table setcos_atrs[] = {
 	/* FINEID 2264 (EIDApplet/7816-15, OPK/EMV/AVANT) */
 	{ "3b:6e:00:00:00:62:00:00:57:41:56:41:4e:54:10:81:90:00", NULL, NULL, SC_CARD_TYPE_SETCOS_FINEID_V2, 0, NULL },
 	{ "3b:7b:94:00:00:80:62:11:51:56:46:69:6e:45:49:44", NULL, NULL, SC_CARD_TYPE_SETCOS_FINEID_V2, 0, NULL },
+	/* FINEID cards 1.3.2011 with Samsung chips (round connector) that supports 2048 bit keys. */
+	{ "3b:7b:94:00:00:80:62:12:51:56:46:69:6e:45:49:44", NULL, NULL, SC_CARD_TYPE_SETCOS_FINEID_V2_2048, 0, NULL },
+	/* FINEID card for organisations, chip unknown. */
+	{ "3b:7b:18:00:00:80:62:01:54:56:46:69:6e:45:49:44", NULL, NULL, SC_CARD_TYPE_SETCOS_FINEID_V2, _FINEID_BROKEN_SELECT_FLAG, NULL },
 	/* Swedish NIDEL card */
 	{ "3b:9f:94:80:1f:c3:00:68:10:44:05:01:46:49:53:45:31:c8:07:90:00:18", NULL, NULL, SC_CARD_TYPE_SETCOS_NIDEL, 0, NULL },
 	/* Setcos 4.4.1 */
@@ -101,7 +107,7 @@ static int setcos_match_card(sc_card_t *card)
 	if (i < 0) {
 		/* Unknown card, but has the FinEID application for sure */
 		if (match_hist_bytes(card, "FinEID", 0)) {
-			card->type = SC_CARD_TYPE_SETCOS_FINEID_V2;
+			card->type = SC_CARD_TYPE_SETCOS_FINEID_V2_2048;
 			return 1;
 		}
 		if (match_hist_bytes(card, "FISE", 0)) {
@@ -162,7 +168,7 @@ static int setcos_init(sc_card_t *card)
 
 	switch (card->type) {
 	case SC_CARD_TYPE_SETCOS_FINEID:
-	case SC_CARD_TYPE_SETCOS_FINEID_V2:
+	case SC_CARD_TYPE_SETCOS_FINEID_V2_2048:
 	case SC_CARD_TYPE_SETCOS_NIDEL:
 		card->cla = 0x00;
 		select_pkcs15_app(card);
@@ -187,8 +193,7 @@ static int setcos_init(sc_card_t *card)
 
 	switch (card->type) {
 	case SC_CARD_TYPE_SETCOS_PKI:
-	case SC_CARD_TYPE_SETCOS_FINEID:
-	case SC_CARD_TYPE_SETCOS_FINEID_V2:
+	case SC_CARD_TYPE_SETCOS_FINEID_V2_2048:
 		{
 			unsigned long flags;
 
@@ -196,6 +201,7 @@ static int setcos_init(sc_card_t *card)
 			flags |= SC_ALGORITHM_RSA_HASH_NONE | SC_ALGORITHM_RSA_HASH_SHA1;
 
 			_sc_card_add_rsa_alg(card, 1024, flags, 0);
+			_sc_card_add_rsa_alg(card, 2048, flags, 0);
 		}
 		break;
 	case SC_CARD_TYPE_SETCOS_44:
@@ -590,6 +596,7 @@ static int setcos_set_security_env2(sc_card_t *card,
 	case SC_SEC_OPERATION_SIGN:
 		/* Should be 0x41 */
 		apdu.p1 = ((card->type == SC_CARD_TYPE_SETCOS_FINEID_V2) ||
+		           (card->type == SC_CARD_TYPE_SETCOS_FINEID_V2_2048) ||
 		           (card->type == SC_CARD_TYPE_SETCOS_44) ||
 			   (card->type == SC_CARD_TYPE_SETCOS_NIDEL) || 
 			   SETCOS_IS_EID_APPLET(card)) ? 0x41 : 0x81;
@@ -673,7 +680,7 @@ static int setcos_set_security_env(sc_card_t *card,
 		switch (card->type) {
 		case SC_CARD_TYPE_SETCOS_PKI:
 		case SC_CARD_TYPE_SETCOS_FINEID:
-		case SC_CARD_TYPE_SETCOS_FINEID_V2:
+		case SC_CARD_TYPE_SETCOS_FINEID_V2_2048:
 		case SC_CARD_TYPE_SETCOS_NIDEL:
 		case SC_CARD_TYPE_SETCOS_44:
 		case SC_CARD_TYPE_SETCOS_EID_V2_0:
@@ -901,6 +908,9 @@ static int setcos_select_file(sc_card_t *card,
 	int r;
 
 	r = iso_ops->select_file(card, in_path, file);
+	/* Certain FINeID cards for organisations return 6A88 instead of 6A82 for missing files */
+	if (card->flags & _FINEID_BROKEN_SELECT_FLAG && r == SC_ERROR_DATA_OBJECT_NOT_FOUND)
+		return SC_ERROR_FILE_NOT_FOUND;
 	if (r)
 		return r;
 	if (file != NULL) {
@@ -1131,9 +1141,7 @@ static struct sc_card_driver *sc_get_driver(void)
 	return &setcos_drv;
 }
 
-#if 1
 struct sc_card_driver *sc_get_setcos_driver(void)
 {
 	return sc_get_driver();
 }
-#endif
