@@ -21,19 +21,22 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "internal.h"
-#include "pkcs15.h"
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-
-#include "esteid.h"
-#include <compat_strlcpy.h>
-
-#ifdef ENABLE_ICONV
-#include <iconv.h>
+#ifdef ENABLE_OPENSSL
+#include <openssl/x509v3.h>
 #endif
 
+#include "common/compat_strlcpy.h"
+#include "common/compat_strlcat.h"
+
+#include "internal.h"
+#include "pkcs15.h"
+#include "esteid.h"
+
 int sc_pkcs15emu_esteid_init_ex(sc_pkcs15_card_t *, sc_pkcs15emu_opt_t *);
 
 static void
@@ -51,9 +54,8 @@ select_esteid_df (sc_card_t * card)
 	int r;
 	sc_path_t tmppath;
 	sc_format_path ("3F00EEEE", &tmppath);
-	tmppath.type = SC_PATH_TYPE_PATH;
 	r = sc_select_file (card, &tmppath, NULL);
-	SC_TEST_RET (card->ctx, r, "esteid select DF failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "esteid select DF failed");
 	return r;
 }
 
@@ -61,70 +63,27 @@ static int
 sc_pkcs15emu_esteid_init (sc_pkcs15_card_t * p15card)
 {
 	sc_card_t *card = p15card->card;
-#ifdef ENABLE_ICONV
-	iconv_t iso_utf;
-	char *inptr, *outptr;
-	size_t inbytes, outbytes, result;
-	unsigned char label[64], name1[32], name2[32];
-#endif
 	unsigned char buff[128];
-	int r, i, flags;
+	int r, i;
 	sc_path_t tmppath;
 
-	set_string (&p15card->label, "ID-kaart");
-	set_string (&p15card->manufacturer_id, "AS Sertifitseerimiskeskus");
+	set_string (&p15card->tokeninfo->label, "ID-kaart");
+	set_string (&p15card->tokeninfo->manufacturer_id, "AS Sertifitseerimiskeskus");
 
 	/* Select application directory */
 	sc_format_path ("3f00eeee5044", &tmppath);
-	tmppath.type = SC_PATH_TYPE_PATH;
 	r = sc_select_file (card, &tmppath, NULL);
-	SC_TEST_RET (card->ctx, r, "select esteid PD failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "select esteid PD failed");
 
 	/* read the serial (document number) */	
 	r = sc_read_record (card, SC_ESTEID_PD_DOCUMENT_NR, buff, sizeof(buff), SC_RECORD_BY_REC_NR);
-	SC_TEST_RET (card->ctx, r, "read document number failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "read document number failed");
 	buff[r] = '\0';
-	set_string (&p15card->serial_number, (const char *) buff);
+	set_string (&p15card->tokeninfo->serial_number, (const char *) buff);
 
-#ifdef ENABLE_ICONV
-	/* Read the name of the cardholder and convert it into UTF-8 */
-	iso_utf  = iconv_open ("UTF-8", "ISO-8859-1");
-	if (iso_utf == (iconv_t) -1)
-		return SC_ERROR_INTERNAL;
-	
-	r = sc_read_record (card, SC_ESTEID_PD_GIVEN_NAMES1, buff, sizeof(buff), SC_RECORD_BY_REC_NR);
-	SC_TEST_RET (card->ctx, r, "read name1 failed");
-	inptr = buff;
-	outptr = name1;
-	inbytes = r;
-	outbytes = 32;
-	result = iconv(iso_utf, &inptr, &inbytes, &outptr, &outbytes);
-	if (result == (size_t) -1)
-		return SC_ERROR_INTERNAL;	
-	*outptr = '\0';
-	
-	r = sc_read_record (card, SC_ESTEID_PD_SURNAME, buff, sizeof(buff), SC_RECORD_BY_REC_NR);
-	SC_TEST_RET (card->ctx, r, "read name2 failed");
-	inptr = buff;
-	outptr = name2;
-	inbytes = r;
-	outbytes = 32;
-	result = iconv(iso_utf, &inptr, &inbytes, &outptr, &outbytes);
-	if (result == (size_t) -1)
-		return SC_ERROR_INTERNAL;
-	*outptr = '\0';
-	
-	snprintf(label, sizeof(label), "%s %s", name1, name2);
-	set_string (&p15card->label, label);
-#endif
-	p15card->flags = SC_PKCS15_CARD_FLAG_PRN_GENERATION
-	                 | SC_PKCS15_CARD_FLAG_EID_COMPLIANT
-	                 | SC_PKCS15_CARD_FLAG_READONLY;
-
-	/* EstEID uses 1024b RSA */
-	card->algorithm_count = 0;
-	flags = SC_ALGORITHM_RSA_PAD_PKCS1;
-	_sc_card_add_rsa_alg (card, 1024, flags, 0);
+	p15card->tokeninfo->flags = SC_PKCS15_TOKEN_PRN_GENERATION
+				  | SC_PKCS15_TOKEN_EID_COMPLIANT
+				  | SC_PKCS15_TOKEN_READONLY;
 
 	/* add certificates */
 	for (i = 0; i < 2; i++) {
@@ -149,6 +108,50 @@ sc_pkcs15emu_esteid_init (sc_pkcs15_card_t * p15card)
 		r = sc_pkcs15emu_add_x509_cert(p15card, &cert_obj, &cert_info);
 		if (r < 0)
 			return SC_ERROR_INTERNAL;
+#ifdef ENABLE_OPENSSL
+		if (i == 0) {
+			BIO *mem = NULL;
+			X509 *x509 = NULL;
+			sc_pkcs15_cert_t *cert;
+			char cardholder_name[64];
+			unsigned char *tmp = NULL;
+			r = sc_pkcs15_read_certificate(p15card, &cert_info, &cert);
+			if (r == SC_SUCCESS) {
+				mem = BIO_new_mem_buf(cert->data, cert->data_len);
+				if (!mem)
+					return SC_ERROR_INTERNAL;
+				x509 = d2i_X509_bio(mem, NULL);
+				BIO_free(mem);
+				if (!x509)
+					return SC_ERROR_INTERNAL;
+				r = X509_NAME_get_index_by_NID(X509_get_subject_name(x509), NID_commonName, -1);
+				if (r >= 0) {
+					X509_NAME_ENTRY *ne;
+					ASN1_STRING *a_str;
+					ne = X509_NAME_get_entry(X509_get_subject_name(x509), r);
+					if (!ne) {
+						X509_free(x509);
+						return SC_ERROR_INTERNAL;
+					}
+					a_str = X509_NAME_ENTRY_get_data(ne);
+					if (!a_str) {
+						X509_free(x509);
+						return SC_ERROR_INTERNAL;
+					}
+					r = ASN1_STRING_to_UTF8(&tmp, a_str);
+					if (r > 0) {
+						if ((unsigned)r > sizeof(cardholder_name) - 1)
+							r = sizeof(cardholder_name) -1;
+						memcpy(cardholder_name, tmp, r);
+						cardholder_name[r] = '\0';
+						set_string(&p15card->tokeninfo->label, cardholder_name);
+						OPENSSL_free(tmp);
+					}
+				}
+				X509_free(x509);
+			}
+		}
+#endif
 	}
 
 	/* the file with key pin info (tries left) */
@@ -170,28 +173,30 @@ sc_pkcs15emu_esteid_init (sc_pkcs15_card_t * p15card)
 		static const int esteid_pin_authid[3] = {1, 2, 3};
 		static const int esteid_pin_flags[3] = {0, 0, SC_PKCS15_PIN_FLAG_UNBLOCKING_PIN};
 		
-		struct sc_pkcs15_pin_info pin_info;
+		struct sc_pkcs15_auth_info pin_info;
 		struct sc_pkcs15_object pin_obj;
 
 		memset(&pin_info, 0, sizeof(pin_info));
 		memset(&pin_obj, 0, sizeof(pin_obj));
 		
 		/* read the number of tries left for the PIN */
-		r = sc_read_record (card, i + 1, buff, 128, SC_RECORD_BY_REC_NR);
+		r = sc_read_record (card, i + 1, buff, sizeof(buff), SC_RECORD_BY_REC_NR);
 		if (r < 0)
 			return SC_ERROR_INTERNAL;
 		tries_left = buff[5];
 		
 		pin_info.auth_id.len = 1;
 		pin_info.auth_id.value[0] = esteid_pin_authid[i];
-		pin_info.reference = esteid_pin_ref[i];
-		pin_info.flags = esteid_pin_flags[i];
-		pin_info.type = SC_PKCS15_PIN_TYPE_ASCII_NUMERIC;
-		pin_info.min_length = esteid_pin_min[i];
-		pin_info.stored_length = 12;
-		pin_info.max_length = 12;
-		pin_info.pad_char = '\0';
+		pin_info.auth_type = SC_PKCS15_PIN_AUTH_TYPE_PIN;	
+		pin_info.attrs.pin.reference = esteid_pin_ref[i];
+		pin_info.attrs.pin.flags = esteid_pin_flags[i];
+		pin_info.attrs.pin.type = SC_PKCS15_PIN_TYPE_ASCII_NUMERIC;
+		pin_info.attrs.pin.min_length = esteid_pin_min[i];
+		pin_info.attrs.pin.stored_length = 12;
+		pin_info.attrs.pin.max_length = 12;
+		pin_info.attrs.pin.pad_char = '\0';
 		pin_info.tries_left = (int)tries_left;
+		pin_info.max_tries = 3;
 
 		strlcpy(pin_obj.label, esteid_pin_names[i], sizeof(pin_obj.label));
 		pin_obj.flags = esteid_pin_flags[i];
@@ -213,10 +218,7 @@ sc_pkcs15emu_esteid_init (sc_pkcs15_card_t * p15card)
 		static int prkey_usage[2] = {
 			SC_PKCS15_PRKEY_USAGE_ENCRYPT
 			| SC_PKCS15_PRKEY_USAGE_DECRYPT
-			| SC_PKCS15_PRKEY_USAGE_SIGN
-			| SC_PKCS15_PRKEY_USAGE_SIGNRECOVER
-			| SC_PKCS15_PRKEY_USAGE_WRAP
-			| SC_PKCS15_PRKEY_USAGE_UNWRAP,
+			| SC_PKCS15_PRKEY_USAGE_SIGN,
 			SC_PKCS15_PRKEY_USAGE_NONREPUDIATION};
 			
 		static const char *prkey_name[2] = {
@@ -234,7 +236,10 @@ sc_pkcs15emu_esteid_init (sc_pkcs15_card_t * p15card)
 		prkey_info.usage  = prkey_usage[i];
 		prkey_info.native = 1;
 		prkey_info.key_reference = i + 1;
-		prkey_info.modulus_length= 1024;
+		if (card->type == SC_CARD_TYPE_MCRD_ESTEID_V30)
+			prkey_info.modulus_length = 2048;
+		else
+			prkey_info.modulus_length = 1024;	
 
 		strlcpy(prkey_obj.label, prkey_name[i], sizeof(prkey_obj.label));
 		prkey_obj.auth_id.len = 1;
@@ -246,14 +251,16 @@ sc_pkcs15emu_esteid_init (sc_pkcs15_card_t * p15card)
 		if (r < 0)
 			return SC_ERROR_INTERNAL;
 	}
+
 	return SC_SUCCESS;
 }
 
 static int esteid_detect_card(sc_pkcs15_card_t *p15card)
 {
-	if (p15card->card->type == SC_CARD_TYPE_MCRD_ESTEID)
+	if (is_esteid_card(p15card->card))
 		return SC_SUCCESS;
-	return SC_ERROR_WRONG_CARD;
+	else		
+		return SC_ERROR_WRONG_CARD;
 }
 
 int sc_pkcs15emu_esteid_init_ex(sc_pkcs15_card_t *p15card,
