@@ -344,21 +344,22 @@ static int itacns_add_pin(sc_pkcs15_card_t *p15card,
 	sc_path_t *path,
 	int flags)
 {
-	struct sc_pkcs15_pin_info pin_info;
+	struct sc_pkcs15_auth_info pin_info;
 	struct sc_pkcs15_object pin_obj;
 
 	SC_FUNC_CALLED(p15card->card->ctx, 1);
 
 	memset(&pin_info, 0, sizeof(pin_info));
+	pin_info.auth_type = SC_PKCS15_PIN_AUTH_TYPE_PIN;
 	pin_info.auth_id.len = 1;
 	pin_info.auth_id.value[0] = id;
-	pin_info.reference = reference;
-	pin_info.flags = flags;
-	pin_info.type = SC_PKCS15_PIN_TYPE_ASCII_NUMERIC;
-	pin_info.min_length = 5;
-	pin_info.stored_length = 8;
-	pin_info.max_length = 8;
-	pin_info.pad_char = 0xff;
+	pin_info.attrs.pin.reference = reference;
+	pin_info.attrs.pin.flags = flags;
+	pin_info.attrs.pin.type = SC_PKCS15_PIN_TYPE_ASCII_NUMERIC;
+	pin_info.attrs.pin.min_length = 5;
+	pin_info.attrs.pin.stored_length = 8;
+	pin_info.attrs.pin.max_length = 8;
+	pin_info.attrs.pin.pad_char = 0xff;
 	if(path)
         pin_info.path = *path;
 
