@@ -333,152 +333,7 @@ int sc_pkcs15_encode_prkdf_entry(sc_context_t *ctx,
 	return r;
 }
 
-/*
- * Store private keys on the card, encrypted
- */
-static const struct sc_asn1_entry	c_asn1_dsa_prkey_obj[] = {
-	{ "privateKey", SC_ASN1_OCTET_STRING, SC_ASN1_TAG_INTEGER, SC_ASN1_ALLOC, NULL, NULL },
-	{ NULL, 0, 0, 0, NULL, NULL }
-};
-
-static int
-sc_pkcs15_encode_prkey_dsa(sc_context_t *ctx,
-		struct sc_pkcs15_prkey_dsa *key,
-		u8 **buf, size_t *buflen)
-{
-	struct sc_asn1_entry	asn1_dsa_prkey_obj[2];
-
-	sc_copy_asn1_entry(c_asn1_dsa_prkey_obj, asn1_dsa_prkey_obj);
-	sc_format_asn1_entry(asn1_dsa_prkey_obj + 0,
-			key->priv.data, &key->priv.len, 1);
-
-	return sc_asn1_encode(ctx, asn1_dsa_prkey_obj, buf, buflen);
-}
-
-static int
-sc_pkcs15_decode_prkey_dsa(sc_context_t *ctx,
-		struct sc_pkcs15_prkey_dsa *key,
-		const u8 *buf, size_t buflen)
-{
-	struct sc_asn1_entry	asn1_dsa_prkey_obj[2];
-
-	sc_copy_asn1_entry(c_asn1_dsa_prkey_obj, asn1_dsa_prkey_obj);
-	sc_format_asn1_entry(asn1_dsa_prkey_obj + 0,
-			&key->priv.data, &key->priv.len, 0);
-
-	return sc_asn1_decode(ctx, asn1_dsa_prkey_obj, buf, buflen, NULL, NULL);
-}
-
-int
-sc_pkcs15_encode_prkey(sc_context_t *ctx,
-		struct sc_pkcs15_prkey *key,
-		u8 **buf, size_t *len)
-{
-	if (key->algorithm == SC_ALGORITHM_DSA)
-		return sc_pkcs15_encode_prkey_dsa(ctx, &key->u.dsa, buf, len);
-	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Cannot encode private key type %u.",
-			key->algorithm);
-	return SC_ERROR_NOT_SUPPORTED;
-}
-
-int
-sc_pkcs15_decode_prkey(sc_context_t *ctx,
-		struct sc_pkcs15_prkey *key,
-		const u8 *buf, size_t len)
-{
-	if (key->algorithm == SC_ALGORITHM_DSA)
-		return sc_pkcs15_decode_prkey_dsa(ctx, &key->u.dsa, buf, len);
-	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Cannot decode private key type %u.",
-			key->algorithm);
-	return SC_ERROR_NOT_SUPPORTED;
-}
-
-int
-sc_pkcs15_read_prkey(struct sc_pkcs15_card *p15card,
-		const struct sc_pkcs15_object *obj,
-		const char *passphrase,
-		struct sc_pkcs15_prkey **out)
-{
-	sc_context_t *ctx = p15card->card->ctx;
-	struct sc_pkcs15_prkey_info *info;
-	struct sc_pkcs15_prkey key;
-	sc_path_t path;
-	u8 *data = NULL;
-	size_t len;
-	int r;
-
-	memset(&key, 0, sizeof(key));
-	switch (obj->type) {
-	case SC_PKCS15_TYPE_PRKEY_RSA:
-		key.algorithm = SC_ALGORITHM_RSA;
-		break;
-	case SC_PKCS15_TYPE_PRKEY_DSA:
-		key.algorithm = SC_ALGORITHM_DSA;
-		break;
-	default:
-		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unsupported object type.");
-		return SC_ERROR_NOT_SUPPORTED;
-	}
-	info = (struct sc_pkcs15_prkey_info *) obj->data;
-	if (info->native) {
-		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Private key is native, will not read.");
-		return SC_ERROR_NOT_ALLOWED;
-	}
-
-	path = info->path;
-	if (path.type == SC_PATH_TYPE_PATH_PROT)
-		path.type = SC_PATH_TYPE_PATH;
-
-	r = sc_pkcs15_read_file(p15card, &path, &data, &len, NULL);
-	if (r < 0) {
-		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unable to read private key file.");
-		return r;
-	}
-
-	/* Is this a protected file? */
-	if (info->path.type == SC_PATH_TYPE_PATH_PROT) {
-		u8 *clear;
-		size_t clear_len;
-
-		if (passphrase == NULL) {
-			r = SC_ERROR_PASSPHRASE_REQUIRED;
-			goto fail;
-		}
-		r = sc_pkcs15_unwrap_data(ctx,
-				passphrase,
-				data, len,
-				&clear, &clear_len);
-		if (r < 0)  {
-			sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Failed to unwrap privat key.");
-			goto fail;
-		}
-		free(data);
-		data = clear;
-		len = clear_len;
-	}
-
-	r = sc_pkcs15_decode_prkey(ctx, &key, data, len);
-	if (r < 0) {
-		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unable to decode private key");
-		goto fail;
-	}
-
-	*out = malloc(sizeof(key));
-	if (*out == NULL) {
-		r = SC_ERROR_OUT_OF_MEMORY;
-		goto fail;
-	}
-
-	**out = key;
-	free(data);
-	return 0;
-
-fail:	if (data)
-		free(data);
-	return r;
-}
-
-void
+static void
 sc_pkcs15_erase_prkey(struct sc_pkcs15_prkey *key)
 {
 	assert(key != NULL);
