@@ -24,7 +24,6 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ltdl.h>
 
 #include "internal.h"
 #include "ctbcs.h"
@@ -252,8 +251,8 @@ static int ctapi_connect(sc_reader_t *reader)
 	lr -= 2;
 	if (lr > SC_MAX_ATR_SIZE)
 		return SC_ERROR_INTERNAL;
-	memcpy(reader->atr, rbuf, lr);
-	reader->atr_len = lr;
+	reader->atr.len = lr;
+	memcpy(reader->atr.value, rbuf, lr);
 	r = _sc_parse_atr(reader);
 
 #if 0	
@@ -350,19 +349,19 @@ static int ctapi_load_module(sc_context_t *ctx,
 	}
 
 	val = conf->name->data;
-	dlh = lt_dlopen(val);
+	dlh = sc_dlopen(val);
 	if (!dlh) {
-		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unable to open shared library '%s': %s\n", val, lt_dlerror());
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unable to open shared library '%s': %s\n", val, sc_dlerror());
 		return -1;
 	}
 
-	funcs.CT_init = (CT_INIT_TYPE *) lt_dlsym(dlh, "CT_init");
+	funcs.CT_init = (CT_INIT_TYPE *) sc_dlsym(dlh, "CT_init");
 	if (!funcs.CT_init)
 		goto symerr;
-	funcs.CT_close = (CT_CLOSE_TYPE *) lt_dlsym(dlh, "CT_close");
+	funcs.CT_close = (CT_CLOSE_TYPE *) sc_dlsym(dlh, "CT_close");
 	if (!funcs.CT_close)
 		goto symerr;
-	funcs.CT_data = (CT_DATA_TYPE *) lt_dlsym(dlh, "CT_data");
+	funcs.CT_data = (CT_DATA_TYPE *) sc_dlsym(dlh, "CT_data");
 	if (!funcs.CT_data)
 		goto symerr;
 
@@ -495,7 +494,7 @@ static int ctapi_load_module(sc_context_t *ctx,
 	return 0;
 symerr:
 	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unable to resolve CT-API symbols.\n");
-	lt_dlclose(dlh);
+	sc_dlclose(dlh);
 	return -1;
 }
 
@@ -540,7 +539,7 @@ static int ctapi_finish(sc_context_t *ctx)
 			struct ctapi_module *mod = &priv->modules[i];
 			
 			free(mod->name);
-			lt_dlclose(mod->dlhandle);
+			sc_dlclose(mod->dlhandle);
 		}
 		if (priv->module_count)
 			free(priv->modules);
@@ -563,6 +562,7 @@ struct sc_reader_driver * sc_get_ctapi_driver(void)
 	ctapi_ops.connect = ctapi_connect;
 	ctapi_ops.disconnect = ctapi_disconnect;
 	ctapi_ops.perform_verify = ctbcs_pin_cmd;
+	ctapi_ops.use_reader = NULL;
 	
 	return &ctapi_drv;
 }
