@@ -86,6 +86,8 @@ struct pcsc_private_data {
 
 	DWORD pin_properties_ioctl;
 
+	DWORD get_tlv_properties;
+
 	int locked;
 };
 
@@ -284,8 +286,9 @@ static int refresh_attributes(sc_reader_t *reader)
 
 	rv = priv->gpriv->SCardGetStatusChange(priv->gpriv->pcsc_ctx, 0, &priv->reader_state, 1);
 
-	if (rv != SCARD_S_SUCCESS && rv != (LONG)SCARD_E_TIMEOUT) {
+	if (rv != SCARD_S_SUCCESS) {
 		if (rv == (LONG)SCARD_E_TIMEOUT) {
+			/* Timeout, no change from previous recorded state. Make sure that changed flag is not set. */
 			reader->flags &= ~SC_READER_CARD_CHANGED;
 			SC_FUNC_RETURN(reader->ctx, SC_LOG_DEBUG_VERBOSE, SC_SUCCESS);
 		}
@@ -395,7 +398,8 @@ static int check_forced_protocol(sc_context_t *ctx, struct sc_atr *atr, DWORD *p
 
 static int pcsc_reconnect(sc_reader_t * reader, DWORD action)
 {
-	DWORD active_proto, tmp, protocol = SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1;
+	DWORD active_proto = opensc_proto_to_pcsc(reader->active_protocol),
+		  tmp, protocol = SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1;
 	LONG rv;
 	struct pcsc_private_data *priv = GET_PRIV_DATA(reader);
 	int r;
@@ -789,6 +793,8 @@ static void detect_reader_features(sc_reader_t *reader, SCARDHANDLE card_handle)
 			priv->modify_ioctl_finish = ntohl(pcsc_tlv[i].value);
 		} else if (pcsc_tlv[i].tag == FEATURE_IFD_PIN_PROPERTIES) {
 			priv->pin_properties_ioctl = ntohl(pcsc_tlv[i].value);
+		} else if (pcsc_tlv[i].tag == FEATURE_GET_TLV_PROPERTIES)  {
+			priv->get_tlv_properties = ntohl(pcsc_tlv[i].value);
 		} else {
 			sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Reader feature %02x is not supported", pcsc_tlv[i].tag);
 		}
@@ -854,7 +860,7 @@ static void detect_reader_features(sc_reader_t *reader, SCARDHANDLE card_handle)
 static int pcsc_detect_readers(sc_context_t *ctx)
 {
 	struct pcsc_global_private_data *gpriv = (struct pcsc_global_private_data *) ctx->reader_drv_data;
-	DWORD active_proto, reader_buf_size;
+	DWORD active_proto, reader_buf_size = 0;
 	SCARDHANDLE card_handle;
 	LONG rv;
 	char *reader_buf = NULL, *reader_name;
@@ -1029,6 +1035,7 @@ static int pcsc_wait_for_event(sc_context_t *ctx, unsigned int event_mask, sc_re
 	size_t i;
 	unsigned int num_watch;
 	int r = SC_ERROR_INTERNAL;
+	DWORD dwtimeout;
 
 	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_NORMAL);
 
@@ -1167,10 +1174,12 @@ static int pcsc_wait_for_event(sc_context_t *ctx, unsigned int event_mask, sc_re
 
 		/* Set the timeout if caller wants to time out */
 		if (timeout == -1) {
-			timeout = INFINITE;
+			dwtimeout = INFINITE;
 		}
+		else
+			dwtimeout = timeout;
 
-		rv = gpriv->SCardGetStatusChange(gpriv->pcsc_wait_ctx, timeout, rgReaderStates, num_watch);
+		rv = gpriv->SCardGetStatusChange(gpriv->pcsc_wait_ctx, dwtimeout, rgReaderStates, num_watch);
 
 		if (rv == (LONG) SCARD_E_CANCELLED) {
 			/* C_Finalize was called, events don't matter */
@@ -1427,6 +1436,91 @@ static int part10_build_modify_pin_block(struct sc_reader *reader, u8 * buf, siz
 	return SC_SUCCESS;
 }
 
+/* Find a given PCSC v2 part 10 property */
+static int
+part10_find_property_by_tag(unsigned char buffer[], int length,
+	int tag_searched)
+{
+	unsigned char *p;
+	int found = 0, len, value = -1;
+
+	p = buffer;
+	while (p-buffer < length)
+	{
+		if (*p++ == tag_searched)
+		{
+			found = 1;
+			break;
+		}
+
+		/* go to next tag */
+		len = *p++;
+		p += len;
+	}
+
+	if (found)
+	{
+		len = *p++;
+
+		switch(len)
+		{
+			case 1:
+				value = *p;
+				break;
+			case 2:
+				value = *p + (*(p+1)<<8);
+				break;
+			case 4:
+				value = *p + (*(p+1)<<8) + (*(p+2)<<16) + (*(p+3)<<24);
+				break;
+			default:
+				value = -1;
+		}
+	}
+
+	return value;
+} /* part10_find_property_by_tag */
+
+/* Make sure the pin min and max are supported by the reader
+ * and fix the values if needed */
+static int
+part10_check_pin_min_max(sc_reader_t *reader, struct sc_pin_cmd_data *data)
+{
+	int r;
+	unsigned char buffer[256];
+	size_t length = sizeof buffer;
+	struct pcsc_private_data *priv = GET_PRIV_DATA(reader);
+
+	r = pcsc_internal_transmit(reader, NULL, 0, buffer, &length,
+		priv->get_tlv_properties);
+	SC_TEST_RET(reader->ctx, SC_LOG_DEBUG_NORMAL, r,
+		"PC/SC v2 part 10: Get TLV properties failed!");
+
+	/* minimum pin size */
+	r = part10_find_property_by_tag(buffer, length,
+		PCSCv2_PART10_PROPERTY_bMinPINSize);
+	if (r >= 0)
+	{
+		unsigned int value = r;
+
+		if (data->pin1.min_length < value)
+			data->pin1.min_length = r;
+	}
+
+	/* maximum pin size */
+	r = part10_find_property_by_tag(buffer, length,
+		PCSCv2_PART10_PROPERTY_bMaxPINSize);
+	if (r >= 0)
+	{
+		unsigned int value = r;
+
+		if (data->pin1.max_length > value)
+			data->pin1.max_length = r;
+	}
+
+	return 0;
+}
+
 /* Do the PIN command */
 static int
 pcsc_pin_cmd(sc_reader_t *reader, struct sc_pin_cmd_data *data)
@@ -1457,6 +1551,7 @@ pcsc_pin_cmd(sc_reader_t *reader, struct sc_pin_cmd_data *data)
 			sc_debug(reader->ctx, SC_LOG_DEBUG_NORMAL, "Pinpad reader does not support verification!");
 			return SC_ERROR_NOT_SUPPORTED;
 		}
+		part10_check_pin_min_max(reader, data);
 		r = part10_build_verify_pin_block(reader, sbuf, &scount, data);
 		ioctl = priv->verify_ioctl ? priv->verify_ioctl : priv->verify_ioctl_start;
 		break;
@@ -1466,6 +1561,7 @@ pcsc_pin_cmd(sc_reader_t *reader, struct sc_pin_cmd_data *data)
 			sc_debug(reader->ctx, SC_LOG_DEBUG_NORMAL, "Pinpad reader does not support modification!");
 			return SC_ERROR_NOT_SUPPORTED;
 		}
+		part10_check_pin_min_max(reader, data);
 		r = part10_build_modify_pin_block(reader, sbuf, &scount, data);
 		ioctl = priv->modify_ioctl ? priv->modify_ioctl : priv->modify_ioctl_start;
 		break;
