@@ -92,33 +92,38 @@ muscle_create_pin(sc_profile_t *profile, sc_pkcs15_card_t *p15card,
 	const unsigned char *puk, size_t puk_len)
 {
 	sc_file_t *file;
-	sc_pkcs15_pin_info_t *pin_info = (sc_pkcs15_pin_info_t *) pin_obj->data;
+	sc_pkcs15_auth_info_t *auth_info = (sc_pkcs15_auth_info_t *) pin_obj->data;
 	int r;
+
 	if ((r = sc_select_file(p15card->card, &df->path, &file)) < 0)
 		return r;
 	if ((r = sc_pkcs15init_authenticate(profile, p15card, file, SC_AC_OP_WRITE)) < 0)
 		return r;
-	pin_info->flags &= ~SC_PKCS15_PIN_FLAG_LOCAL;
+
+	auth_info->attrs.pin.flags &= ~SC_PKCS15_PIN_FLAG_LOCAL;
 	return 0;
 }
 
 static int
 muscle_select_pin_reference(sc_profile_t *profike, sc_pkcs15_card_t *p15card,
-		sc_pkcs15_pin_info_t *pin_info)
+		sc_pkcs15_auth_info_t *auth_info)
 {
 	int	preferred;
 
-	if (pin_info->flags & SC_PKCS15_PIN_FLAG_SO_PIN) {
+	if (auth_info->auth_type != SC_PKCS15_PIN_AUTH_TYPE_PIN)
+		return SC_ERROR_OBJECT_NOT_VALID;
+
+	if (auth_info->attrs.pin.flags & SC_PKCS15_PIN_FLAG_SO_PIN) {
 		preferred = 0;
 	} else {
 		preferred = 1;
 	}
-	if (pin_info->reference <= preferred) {
-		pin_info->reference = preferred;
+	if (auth_info->attrs.pin.reference <= preferred) {
+		auth_info->attrs.pin.reference = preferred;
 		return 0;
 	}
 
-	if (pin_info->reference > 2)
+	if (auth_info->attrs.pin.reference > 2)
 		return SC_ERROR_INVALID_ARGUMENTS;
 
 	/* Caller, please select a different PIN reference */
